@@ -151,6 +151,7 @@ cl_load_1_0 (int fd)
 				(*pc)->next = NULL;
 				(*pc)->state.ip = (ip_t)0;
 				(*pc)->state.socket = (sock_t)-1;
+				(*pc)->state.connected = FALSE;
 				(*pc)->state.hello_pending = TRUE;
 				(*pc)->state.in_buffer = NULL;
 				(*pc)->state.in_buf_size = 0;
@@ -383,7 +384,7 @@ cl_remove_contact (const char *nick)
 					ui_close_win (p);
 				}
 				(*pc) = (*pc)->next;
-				if (p->state.socket != -1)
+				if (p->state.socket >= 0)
 					tr_disconnect (p); 
 				free (p);	
 				cl_save ();
