@@ -37,26 +37,43 @@
 #include "transport.h"
 #include "misc.h"
 
-#define ADD_C 1
-#define DEL_C 2
-#define HELP_C 3
-#define NICK_C 4
-#define PORT_C 5
-#define QUIT_C 6
-#define CLOSE_C 7
-#define INFO_C 8
+#define ADD_C 0
+#define BEEP_C 1
+#define CLOSE_C 2
+#define DELETE_C 3
+#define HELP_C 4
+#define INFO_C 5
+#define NICK_C 6
+#define QUIT_C 7
+#define SET_C 8
+#define COMMANDS_NUM 9
+#define C_LIST 1000
 
+#define BEEP_V 0
+#define LOGGING_V 1
+#define PORT_V 2
+#define TIMESTAMP_V 3
+#define VARS_NUM 4
+
+#define MAX_L 12
 #define MAX_W 10
 
 #define ERR_UNK (ERR_LOCAL -1)
 #define ERR_AMB (ERR_LOCAL -2)
 
 
+char commands [COMMANDS_NUM][MAX_L] = {"add", "beep", "close", "delete", "help",
+                                       "info", "nick", "quit", "set"};
+char vars [VARS_NUM][MAX_L] = {"beep", "logging", "port", "timestamps"};
+
 static int get_words (char **t, const char *str, int max);
 static void free_words (char **t, int max);
-static int command_id (char *str);
-static void help (int command_id);
+static int command_id (char *str, char (*cl)[MAX_L], int num);
+static void help (int c_id, char *opt);
 static void print_contact_info (contact_t *c);
+static void toggle_setting (BOOL *set, char *p);
+static void set_command (char *var, char *p);
+static void set_help (int var_id);
 
 void 
 pa_parse_line (contact_t *win_contact, const char *line)
@@ -65,10 +82,8 @@ pa_parse_line (contact_t *win_contact, const char *line)
 	int co;
 	int co_h;
 	int w;
-	char *ptr;
 	contact_t *ct;
-	port_t port;
-	
+		
 	if (line [0] != '/') {
 		if (win_contact) {
 			pr_send_text (win_contact, line);
@@ -82,14 +97,16 @@ pa_parse_line (contact_t *win_contact, const char *line)
 	//ui_output_err (line);
 	
 	w = get_words (words, line, MAX_W);
-	co = command_id (words [0]+1);
+	co = command_id (words [0]+1, commands, COMMANDS_NUM);
 	
 	switch (co) {
 		case ADD_C:
 			if (w > 2) {
 				ct = malloc (sizeof (contact_t));
-				if (ct == NULL)
+				if (ct == NULL) {
 					dmx_stop ();
+					return;
+				}
 				strncpy (ct->nick, words [1], MAX_NICK-1);
 				ct->nick [MAX_NICK-1]='\0';
 				strncpy (ct->hname, words [2], MAX_HNAME-1);
@@ -102,6 +119,7 @@ pa_parse_line (contact_t *win_contact, const char *line)
 				ct->next = NULL;
 				ct->state.ip = 0;
 				ct->state.socket=-1;
+				ct->state.connected=FALSE;
 				ct->state.hello_pending=TRUE;
 				ct->state.in_buffer=NULL;
 				ct->state.in_buf_size=0;
@@ -115,63 +133,34 @@ pa_parse_line (contact_t *win_contact, const char *line)
 			else
 				ui_output_err ("Not enough parameters.");
 			break;
-		case DEL_C:
-			if (w > 1)
-				cl_remove_contact (words [1]);
-			else
-				ui_output_err ("Not enough parameters.");
-			break;
-		case HELP_C:
+		case BEEP_C:
 			if (w > 1) {
-				co_h = command_id (words [1]);
-				if (co_h == ERR_UNK)
-					ui_output_err ("No help for %s.", 
-							words[1]);
-				else if (co_h == ERR_AMB)
-					ui_output_err ("Ambiguous command: %s", 
-							words[1]);
-				else
-					help (co_h);
-			}
-			else
-				help (0);
-			break;
-		case NICK_C:
-			if (w > 1) {
-				strncpy (cfg.nick, words[1], MAX_NICK-1);
-				cfg.nick [MAX_NICK-1] = '\0';
-				cf_save ();
-				ui_redraw_nick ();
-				pr_send_hello (BROADCAST);
-			}
-			else
-				ui_output_err ("Not enough parameters.");
-			break;
-		case PORT_C:
-			if (w > 1) {
-				port = (port_t) strtoul (words[1], &ptr, 10);
-				if (*ptr != '\0')
-					ui_output_err ("Invalid port number.");
-				else if (cfg.listen_port == port) {
-					ui_output_info ("That's the current "
-					                "listen port.");
+				if (strcmp (words[1], "*") == 0) {
+					pr_send_beep (BROADCAST);
 				}
 				else {
-					cfg.listen_port = port;
-					cf_save ();
-					pr_send_hello (BROADCAST);
-					ui_output_info ("Listen port changed "
-					                "to %u. You need to "
-					                "restart ipchat.",
-					                cfg.listen_port);
+					contact_t *c;
+					c = cl_find_by_nick (words[1]);
+					if (c == NULL) {
+						ui_output_err ("Contact %s not "
+						               "found.",
+							       words[1]);
+					}
+					else {
+						pr_send_beep (c);
+					}
+				}
+			}
+			else {
+				contact_t *c = ui_current_contact ();
+				if (c == NULL) {
+					ui_output_err ("Contact not "
+					               "connected.");
+				}
+				else {
+					pr_send_beep (c);
 				}
 			}
-			else
-				ui_output_info ("You are listening on port %u.",
-				                cfg.listen_port);
-			break;		
-		case QUIT_C:
-			dmx_stop ();
 			break;
 		case CLOSE_C:
 			if (w > 1) {
@@ -187,6 +176,28 @@ pa_parse_line (contact_t *win_contact, const char *line)
 			else
 				ui_close_cur_win ();
 			break;
+		case DELETE_C:
+			if (w > 1)
+				cl_remove_contact (words [1]);
+			else
+				ui_output_err ("Not enough parameters.");
+			break;
+		case HELP_C:
+			if (w > 1) {
+				co_h = command_id (words [1], commands, 
+						   COMMANDS_NUM);
+				if (co_h == ERR_UNK)
+					ui_output_err ("No help for %s.", 
+							words[1]);
+				else if (co_h == ERR_AMB)
+					ui_output_err ("Ambiguous command: %s", 
+							words[1]);
+				else
+					help (co_h, words[2]);
+			}
+			else
+				help (C_LIST, NULL);
+			break;
 		case INFO_C:
 			if (w > 1) {
 				contact_t *c = cl_find_by_nick (words[1]);
@@ -201,6 +212,23 @@ pa_parse_line (contact_t *win_contact, const char *line)
 			else
 				ui_output_err ("Not enough parameters.");
 			break;
+		case NICK_C:
+			if (w > 1) {
+				strncpy (cfg.nick, words[1], MAX_NICK-1);
+				cfg.nick [MAX_NICK-1] = '\0';
+				cf_save ();
+				ui_redraw_nick ();
+				pr_send_hello (BROADCAST);
+			}
+			else
+				ui_output_err ("Not enough parameters.");
+			break;
+		case SET_C:
+			set_command (words[1], words[2]);
+			break;		
+		case QUIT_C:
+			dmx_stop ();
+			break;
 		case ERR_UNK:
 			ui_output_err ("Unknown command: %s", words [0]+1);
 			break;
@@ -252,7 +280,7 @@ free_words (char **t, int max)
 }
 
 static int 
-command_id (char *str)
+command_id (char *str, char (*cl)[MAX_L], int num)
 {
 	int i = 0;
 	int ret = ERR_UNK;
@@ -269,39 +297,18 @@ command_id (char *str)
 		i++;
 	}
 	
-	switch (*c) {
-		case 'a':
-			if (strncmp (c, "add", strlen (c)) == 0)
-				ret = ADD_C;		
-			break;
-		case 'c':
-			if (strncmp (c, "close", strlen (c)) == 0)
-				ret = CLOSE_C;
-			break;
-		case 'd':
-			if (strncmp (c, "delete", strlen (c)) == 0)
-				ret = DEL_C;
-			break;
-		case 'h':
-			if (strncmp (c, "help", strlen (c)) == 0)
-				ret = HELP_C;
-			break;
-		case 'i':
-			if (strncmp (c, "info", strlen (c)) == 0)
-				ret = INFO_C;
-			break;
-		case 'n':
-			if (strncmp (c, "nick", strlen (c)) == 0)
-				ret = NICK_C;
-			break;
-		case 'p':
-			if (strncmp (c, "port", strlen (c)) == 0)
-				ret = PORT_C;
-			break;
-		case 'q':
-			if (strncmp (c, "quit", strlen (c)) == 0)
-				ret = QUIT_C;
-			break;
+	i = 0;
+	while (i < num) {
+		if (strncmp (c, cl[i], strlen (c)) == 0) {
+			if (ret == ERR_UNK)
+				ret = i;
+			else {
+				ret = ERR_AMB;
+				break;
+			}
+		}	
+
+		i++;
 	}
 
 	free (c);
@@ -309,9 +316,11 @@ command_id (char *str)
 }
 
 static void
-help (int command_id)
+help (int c_id, char *opt)
 {
-	switch (command_id) {
+	int v_id;
+	
+	switch (c_id) {
 		case ADD_C:
 			ui_output_info ("");
 			ui_output_info ("ADD <nick> <host> [<port>]");
@@ -320,13 +329,20 @@ help (int command_id)
 			                "given as an IP address or as a "
 			                "hostname.");
 			break;
+		case BEEP_C:
+			ui_output_info ("");
+			ui_output_info ("BEEP [<nick>]");
+			ui_output_info ("");
+			ui_output_info ("Beeps current or specified contact "
+			                "(use \"*\" to beep everybody).");
+			break;
 		case CLOSE_C:
 			ui_output_info ("");
 			ui_output_info ("CLOSE [<nick>]");
 			ui_output_info ("");
 			ui_output_info ("Closes current or specified window.");
 			break;
-		case DEL_C:
+		case DELETE_C:
 			ui_output_info ("");
 			ui_output_info ("DELETE <nick>");
 			ui_output_info ("");
@@ -350,12 +366,31 @@ help (int command_id)
 			ui_output_info ("");
 			ui_output_info ("Changes your nick.");
 			break;
-		case PORT_C:
-			ui_output_info ("");
-			ui_output_info ("PORT [<port>]");
-			ui_output_info ("");
-			ui_output_info ("Shows or changes your listening "
-			                "port.");
+		case SET_C:
+			if (opt == NULL) {
+				ui_output_info ("");
+				ui_output_info ("SET <setting> [<value>]");
+				ui_output_info ("");
+				ui_output_info ("Changes IPchat settings.");
+				ui_output_info ("You can use /HELP SET " 
+						"[<setting>] to get more help "
+					        "about a particular setting.");
+				ui_output_info ("");
+				ui_output_info ("\\b\\3Available settings");
+				ui_output_info ("\\b\\3------------------");
+				set_command (NULL, NULL);
+			}
+			else {
+  				v_id = command_id (opt, vars, VARS_NUM);
+				if (v_id == ERR_UNK)
+					ui_output_err ("Unknown setting: %s.", 
+							opt);
+				else if (v_id == ERR_AMB)
+					ui_output_err ("Ambiguous setting: %s",
+							opt);
+				else
+					set_help (v_id);
+			}
 			break;
 		case QUIT_C:
 			ui_output_info ("");
@@ -363,17 +398,18 @@ help (int command_id)
 			ui_output_info ("");
 			ui_output_info ("Ends your ipchat session.");
 			break;
-		default:
+		case C_LIST:
 			ui_output_info ("");
 			ui_output_info ("\\b\\3Available commands");
 			ui_output_info ("\\b\\3------------------");
 			ui_output_info ("\\bADD");
+			ui_output_info ("\\bBEEP");
 			ui_output_info ("\\bCLOSE");
 			ui_output_info ("\\bDELETE");
 			ui_output_info ("\\bHELP");
 			ui_output_info ("\\bINFO");
 			ui_output_info ("\\bNICK");
-			ui_output_info ("\\bPORT");
+			ui_output_info ("\\bSET");
 			ui_output_info ("\\bQUIT");
 			ui_output_info ("");
 			ui_output_info ("For information on a command use "
@@ -387,6 +423,39 @@ help (int command_id)
 	}
 }
 
+static void
+set_help (int var_id)
+{
+	switch (var_id) {
+		case PORT_V:
+			ui_output_info ("");
+			ui_output_info ("SET PORT [<port>]");
+			ui_output_info ("");
+			ui_output_info ("Shows or changes your listening "
+					"port.");
+			break;
+		case LOGGING_V:
+			ui_output_info ("");
+			ui_output_info ("SET LOGGING [ON|OFF]");
+			ui_output_info ("");
+			ui_output_info ("Activates/deactivates logging "
+			                "(NOT YET IMPLEMENTED).");
+			break;
+		case TIMESTAMP_V:
+ 			ui_output_info ("");
+                        ui_output_info ("SET TIMESTAMPS [ON|OFF]");
+			ui_output_info ("");
+			ui_output_info ("Activates/deactivates timestamps.");					       
+			break;
+		case BEEP_V:
+       	     		ui_output_info ("");	     
+		      	ui_output_info ("SET BEEP [ON|OFF]");
+			ui_output_info ("");
+			ui_output_info ("Activates/deactivates beep.");
+			break;
+	}
+}
+
 static void
 print_contact_info (contact_t *c)
 {
@@ -413,8 +482,118 @@ print_contact_info (contact_t *c)
 	}
 	ui_output_info ("\\b\\2Listen port:   \\0%u", c->port);
 	ui_output_info ("\\b\\2State:         \\0%s",
-	                c->state.socket < 0 ? "Disconnected" :
+	                (!c->state.connected) ? "Disconnected" :
 	                c->state.hello_pending ? "Waiting HELLO..."
 	                                       : "Connected");
 	ui_output_info ("");
 }
+
+static void
+set_command (char *var, char *p)
+{
+	port_t port;
+	char *ptr;
+	BOOL u = FALSE;
+	int i=0;
+	
+	if (p == NULL) {
+		if (var == NULL)
+			var = "\0";
+		else {
+			while (var[i] != '\0') {
+				var[i] = tolower ((int) var[i]);
+     			      	i++;
+			}
+		}
+		
+		if (strncmp (vars[0], var, strlen(var)) == 0) {
+	
+			u = TRUE;
+			if (cfg.beep)
+				ui_output_info ("\\bbeep = ON");
+			else
+				ui_output_info ("\\bbeep = OFF");
+		}
+		if (strncmp (vars[1], var, strlen(var)) == 0) {
+
+			u = TRUE;
+			if (cfg.logging)
+				ui_output_info ("\\blogging = ON (NOT YET "
+				                "IMPLEMENTED)");
+			else
+				ui_output_info ("\\blogging = OFF");
+		}
+	
+		if (strncmp (vars[2], var, strlen(var)) == 0) {
+			u = TRUE;
+			ui_output_info ("\\bport = %u", cfg.listen_port);
+		}
+		if (strncmp (vars[3], var, strlen(var)) == 0) {
+			u = TRUE;
+			if (cfg.timestamps)
+				ui_output_info ("\\btimestamps = ON");
+			else
+				ui_output_info ("\\btimestamps = OFF");
+		}
+	
+		if (!u)
+			ui_output_err ("Unknown setting: %s", var);
+		
+	}
+
+	else {
+		switch (command_id (var, vars, VARS_NUM)) {
+			case BEEP_V:
+				toggle_setting (&cfg.beep, p);
+				break;
+			case LOGGING_V:
+				toggle_setting (&cfg.logging, p);
+				break;
+			case PORT_V:
+				port = (port_t) strtoul (p, &ptr, 10);
+				if (*ptr != '\0')
+					ui_output_err ("Invalid port number.");
+				else if (cfg.listen_port == port) {
+					ui_output_info ("That's the current "
+					                "listen port.");
+				}
+				else {
+					cfg.listen_port = port;
+					pr_send_hello (BROADCAST);
+					ui_output_info ("Listen port changed "
+					                "to %u. You need to "
+					                "restart ipchat.",
+					                cfg.listen_port);
+				}
+				break;
+			case TIMESTAMP_V:
+				toggle_setting (&cfg.timestamps, p);
+				break;
+			case ERR_AMB:
+				ui_output_err ("Ambiguous setting: %s", var);
+				break;
+			case ERR:
+				dmx_stop();
+		} 
+		set_command (var, NULL);
+		cf_save();
+	}
+}
+
+static void
+toggle_setting (BOOL *set, char *p)
+{
+	int i = 0;
+
+	while (p[i] != '\0') {
+		p[i] = tolower ((int) p[i]);
+		i++;
+	}
+		
+	if (strcmp (p, "on") == 0)
+		*set = TRUE;
+	else if (strcmp (p, "off") == 0)
+		*set = FALSE;
+	else
+		ui_output_err ("Value must be ON or OFF");
+}
