@@ -43,6 +43,10 @@ struct msg_header {
 		struct hdr_text {
 		} text;
 
+		/* Beep message. */
+		struct hdr_beep {
+		} beep;
+
 		/* ... */
 	} c;
 } __attribute__((packed));  /* TODO: this is gcc only */
@@ -51,7 +55,7 @@ struct msg_header {
 static struct msg_header header;
 
 /* Values of 'header.type'. */
-enum { MSG_HELLO = 0, MSG_TEXT /* ... */ };
+enum { MSG_HELLO = 0, MSG_TEXT, MSG_BEEP /* ... */ };
 
 /* Macro to calculate how many bytes of 'header' have to be sent or received. */
 #define HEADER_SIZE(t) (sizeof(header.c.t) + sizeof(header) - sizeof(header.c))
@@ -66,6 +70,7 @@ enum { MSG_HELLO = 0, MSG_TEXT /* ... */ };
 static int received_hello (contact_t *contact, struct hdr_hello *hdr,
                            const char *nick, int nick_len);
 static int received_text (contact_t *contact, const char *text, int len);
+static int received_beep (contact_t *contact);
 static void hton_header ();
 static void ntoh_header ();
 static int header_size (uint16_t msg_type);
@@ -90,6 +95,21 @@ pr_send_hello (contact_t *contact)
 	             (BYTE *)cfg.nick, strlen (cfg.nick));
 }
 
+void
+pr_send_beep (contact_t *contact)
+{
+	if (contact != BROADCAST && contact->state.hello_pending) {
+		ui_output_err ("Contact not connected.");
+		return;
+	}
+	header.type = MSG_BEEP;
+	hton_header ();
+	tr_send_msg (contact, (BYTE *)&header, HEADER_SIZE(beep),
+	             (BYTE *)0, 0);
+	ui_output_info ("Beeping %s.", contact == BROADCAST ? "everybody"
+	                                                    : contact->nick);
+}
+
 int
 pr_msg_received (contact_t *contact, const BYTE *msg, int len)
 {
@@ -123,6 +143,10 @@ pr_msg_received (contact_t *contact, const BYTE *msg, int len)
 			                   pl_size);
 			break;
 
+		case MSG_BEEP:
+			r = received_beep (contact);
+			break;
+
 		default:
 			ui_output_err ("BUG: missing type in pr_msg_received.");
 			r = ERR;
@@ -137,8 +161,10 @@ small_size:
 	return ERR;
 
 bad_type:
-	ui_output_err ("Received message of unknown type.");
-	return ERR;
+	ui_output_err ("Received message of unknown type. You may need to "
+	               "update IPChat.");
+	/* return OK because there is no need to disconnect */
+	return OK;
 }
 
 static int
@@ -147,12 +173,19 @@ received_hello (contact_t *contact, struct hdr_hello *hdr, const char *nick,
 {
 	BOOL modif, modif_nick, hello_was_pending;
 
-	if (hdr->version != PROTOCOL_VERSION) {
+	if (hdr->version < PROTOCOL_VERSION) {
 		ui_output_err ("Connection from outdated client! (from %s)",
 		               ipv4_to_string (contact->state.ip));
 		return ERR;
 	}
 
+	if (hdr->version > PROTOCOL_VERSION) {
+		ui_output_err ("You could need to upgrade IPChat, someone "
+		               "(from %s) is using an incompatible newer "
+		               "version.", ipv4_to_string (contact->state.ip));
+		return ERR;
+	}
+
 	if (nick_len > (MAX_NICK - 1)) {
 		ui_output_err ("Contact's nick is too long (from %s).",
 		               ipv4_to_string (contact->state.ip));
@@ -223,6 +256,20 @@ received_text (contact_t *contact, const char *text, int len)
 	return OK;
 }
 
+static int
+received_beep (contact_t *contact)
+{
+	if (cfg.beep) {
+		ui_output_info ("Beep from %s!", contact->nick);
+		ui_beep ();
+	}
+	else {
+		ui_output_info ("Beep from %s! (deactivated)", contact->nick);
+	}
+
+	return OK;
+}
+
 static void
 hton_header ()
 {
@@ -236,6 +283,9 @@ hton_header ()
 		case MSG_TEXT:
 			break;
 
+		case MSG_BEEP:
+			break;
+
 		/* ... */
 	}
 
@@ -257,6 +307,9 @@ ntoh_header ()
 		case MSG_TEXT:
 			break;
 
+		case MSG_BEEP:
+			break;
+
 		/* ... */
 	}
 }
@@ -273,6 +326,9 @@ header_size (uint16_t msg_type)
 		case MSG_TEXT:
 			sz = HEADER_SIZE(text);
 			break;
+		case MSG_BEEP:
+			sz = HEADER_SIZE(beep);
+			break;
 		default:
 			sz = ERR;
 			break;
