@@ -75,15 +75,21 @@ tr_init_server ()
 	dmx_add_input_fd (s, cb_server_data, NULL);
 }
 
-/* TODO should be static? */
 void
 tr_connect (contact_t *contact)
 {
 	sock_t s;
 
-	CHECK (contact->state.ip);
+	CHECK (contact->state.ip && contact->state.socket < 0);
+
+	if (contact->state.connected) {
+		ui_output_err ("Program error in tr_connect. Please report. "
+		               "Connected without a socket!?");
+	}
+
 	s = tcp_connect (contact->state.ip, contact->port);
 	CHECK (s >= 0);
+	contact->state.socket = s;
 	dmx_add_output_fd (s, cb_out_connection, (void *)contact);
 	/* TODO creamos un timeout o usamos el de tcp del kernel? */
 
@@ -119,6 +125,7 @@ tr_disconnect (contact_t *contact)
 	}
 	contact->state.in_buf_size = 0;
 
+	contact->state.connected = FALSE;
 	contact->state.hello_pending = TRUE;
 
 	if (contact->nick[0] != '\0') {
@@ -145,13 +152,13 @@ tr_send_msg (contact_t *contact, const BYTE *header, int hdr_len,
 		   established but hello didn't arrive, the contact
 		   would never know about the new nick */
 		for (contact = contacts;
-		     contact && contact->state.socket < 0;
+		     contact && !contact->state.connected;
 		     contact = contact->next)
 			;
 		if (!contact)
 			return;
 	}
-	else if (contact->state.socket < 0) {
+	else if (!contact->state.connected) {
 		ui_output_err ("Trying to send data to not connected "
 		               "contact. This is a bug, please report it.");
 		return;
@@ -178,7 +185,7 @@ tr_send_msg (contact_t *contact, const BYTE *header, int hdr_len,
 		/* copy full msg to all other connected contacts */
 		for (contact = contact->next; contact;
 		     contact = contact->next) {
-			if (contact->state.socket >= 0) {
+			if (contact->state.connected) {
 				was_void = (contact->state.out_buffer == NULL);
 				p = inc_out_buffer (contact, len_full_msg);
 				CHECK (p);
@@ -201,7 +208,7 @@ tr_resolv (contact_t *contact)
 {
 	struct hostent *he;
 
-	/* connected contacts don't change IP address */
+	/* connected/connecting contacts don't change IP address */
 	if (contact->state.socket < 0) {
 		he = gethostbyname (contact->hname);
 		if (he && he->h_addrtype == AF_INET &&
@@ -249,13 +256,19 @@ cb_server_data (int fd, void *data)
 		c = create_temp_contact (ip);
 		CHECK (c);
 	}
-	else if (c->state.socket >= 0) {
+	else if (c->state.connected) {
 		ui_output_info ("%s reconnected, closing old connection.",
 		                c->nick);
 		tr_disconnect (c);
 	}
+	else if (c->state.socket >= 0) {
+		/* this is a connecting socket, just remove it */
+		dmx_remove_output_fd (c->state.socket);
+		close (c->state.socket);
+	}
 
 	c->state.socket = s;
+	c->state.connected = TRUE;
 	c->state.hello_pending = TRUE; /* TODO it should already be, check it */
 	dmx_add_input_fd (s, cb_net_input, (void *)c);
 	create_hello_timeout (c);
@@ -293,18 +306,21 @@ cb_out_connection (int fd, void *data)
 {
 	contact_t *c = (contact_t *)data;
 
+	if (fd != c->state.socket || c->state.connected) {
+		ui_output_err ("Program error in cb_out_connection. Please "
+		               "report. fd=%d socket=%d.", fd, c->state.socket);
+	}
 	dmx_remove_output_fd (fd);
 
-	/* the contact could have initiated a connection at the same time */
-	if (tcp_connect_result (fd) == OK && c->state.socket < 0) {
-		/* TODO we could check that hello_pending is TRUE */
-		c->state.socket = fd;
+	if (tcp_connect_result (fd) == OK) {
+		c->state.connected = TRUE;
 		dmx_add_input_fd (fd, cb_net_input, data);
 		create_hello_timeout (c);
 		pr_send_hello (c);
 	}
 	else {
 		close (fd);
+		c->state.socket = -1;
 	}
 }
 
@@ -443,6 +459,7 @@ create_temp_contact (ip_t ip)
 	c->next = NULL;
 	c->state.ip = ip;
 	c->state.socket = -1;
+	c->state.connected = FALSE;
 	c->state.hello_pending = TRUE;
 	c->state.in_buffer = NULL;
 	c->state.in_buf_size = 0;
@@ -519,7 +536,7 @@ cb_check_contacts (void *data)
 	contact_t *c;
 
 	for (c = contacts; c; c = c->next) {
-		/* no need to check connected contacts */
+		/* no need to check connected/connecting contacts */
 		if (c->state.socket < 0) {
 			tr_connect (c);
 		}
