@@ -53,8 +53,9 @@
 
 static int cf_load_1_0 (int fd);
 static int cf_save_1_0 (int fd);
+static BOOL check_on (char *str);
 
-config_t cfg = {DEFAULT_NICK, DEFAULT_PORT};
+config_t cfg = {DEFAULT_NICK, DEFAULT_PORT, TRUE, FALSE, FALSE};
 
 int 
 cf_load ()
@@ -136,6 +137,24 @@ cf_load_1_0 (int fd)
 	if (r == 0)
 		cfg.listen_port = (port_t) strtoul (port, NULL, 10);
 	
+	lseek (fd, 0, SEEK_SET); 
+	
+	r = read_tag (fd, NULL, "beep", nick, MAX_NICK);
+	if (r == 0)
+		cfg.beep = check_on (nick);
+	
+	lseek (fd, 0, SEEK_SET); 
+        
+	r = read_tag (fd, NULL, "logging", nick, MAX_NICK);
+        if (r == 0)
+	   	cfg.logging = check_on (nick);
+	
+	lseek (fd, 0, SEEK_SET); 
+        
+	r = read_tag (fd, NULL, "timestamps", nick, MAX_NICK);
+        if (r == 0)
+                cfg.timestamps = check_on (nick);
+	
 	return OK;
 }
 	
@@ -222,12 +241,41 @@ cf_save_1_0 (int fd)
 {
 	char b [PORT_BUF_SIZE];
 	
-	CHECK (write_tag (fd, "nick", cfg.nick, 0) != ERR);
+	if (cfg.beep)
+   		CHECK (write_tag (fd, "beep", "on", 0) != ERR);
+	else
+                CHECK (write_tag (fd, "beep", "off", 0) != ERR);
 	snprintf (b, PORT_BUF_SIZE, "%u", cfg.listen_port);
 	CHECK (write_tag (fd, "listen_port", b, 0) != ERR);
+	if (cfg.logging)
+                CHECK (write_tag (fd, "logging", "on", 0) != ERR);
+        else
+		CHECK (write_tag (fd, "logging", "off", 0) != ERR);
+	CHECK (write_tag (fd, "nick", cfg.nick, 0) != ERR);
+	if (cfg.timestamps)
+		CHECK (write_tag (fd, "timestamps", "on", 0) != ERR);
+	else
+		CHECK (write_tag (fd, "timestamps", "off", 0) != ERR);
+
 	
 	return OK;
 error:
 	return ERR;
 }
 
+static BOOL
+check_on (char *str)
+{
+	char *p = str;
+
+	if (str != NULL) {
+		while (*p != '\0') {
+			*p = tolower (*p);
+			p++;
+		}
+		if (strcmp ("on", str) == 0)
+			return TRUE;
+	}
+	return FALSE;
+}
+
